package dareka;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import dareka.common.Config;
import dareka.common.Logger;
import dareka.processor.impl.Cache;

public class Main {
    // public so that external tools can read.
    public static final String VER_STRING = "NicoCache v0.20";

    // accessor for avoiding static link
    public static String getVersion() {
        return VER_STRING;
    }

    private static Server server;

    public static void stop() {
        if (server != null) {
            server.stop();
        }
    }

    /**
     * @param args
     * @throws IOException
     */
    public static void main(String[] args) throws IOException {
        File configFile = new File("config.properties");
        Config config = new Config();
        if (configFile.exists()) {
            Properties p = new Properties();

            FileInputStream in = new FileInputStream(configFile);
            try { // ensure closing in
                p.load(in);
            } finally {
                in.close();
            }

            String v;

            v = p.getProperty(Config.LISTEN_PORT);
            if (v != null) {
                config.setListenPort(Integer.valueOf(v));
            }

            v = p.getProperty(Config.PROXY_HOST);
            if (v != null) {
                config.setProxyHost(v);
            }

            v = p.getProperty(Config.PROXY_PORT);
            if (v != null) {
                config.setProxyPort(Integer.valueOf(v));
            }

            v = p.getProperty(Config.TITLE);
            if (v != null) {
                config.setTitle(Boolean.valueOf(v));
            }
        } else {
            Properties p = new Properties();
            p.setProperty(Config.LISTEN_PORT, Integer.toString(config
                    .getListenPort()));
            p.setProperty(Config.PROXY_HOST, config.getProxyHost());
            p.setProperty(Config.PROXY_PORT, Integer.toString(config
                    .getProxyPort()));
            p.setProperty(Config.TITLE, Boolean.toString(config.isTitle()));

            FileOutputStream out = new FileOutputStream(configFile);
            try { // ensure closing out
                p.store(out, "NicoCache config file");
            } finally {
                out.close();
            }
        }

        Logger.info(VER_STRING);
        Logger.info("port=" + config.getListenPort());
        if (config.getProxyHost().equals("")) {
            Logger.info("direct mode (no secondary proxy)");
        } else {
            Logger.info("proxy host=" + config.getProxyHost());
            Logger.info("proxy port=" + config.getProxyPort());
        }
        Logger.info("title=" + config.isTitle());

        if (Boolean.getBoolean("dareka.debug")) {
            Logger.info("debug mode");
        }

        Cache.init();
        Cache.cleanup();
        Logger.info("total cache size=%,dbytes\n", Cache.size());

        server = new Server(config);

        server.start();
    }
}
