package dareka.common;

import java.io.PrintWriter;
import java.io.StringWriter;

import dareka.launcher.logview.LogWindow;

public class Logger {
    private static final boolean DEBUG = Boolean.getBoolean("dareka.debug");

    private Logger() {
        // avoid instantiation
    }

    public static void debug(String message) {
        if (DEBUG) {
            System.out.println("DEBUG: " + message);
            LogWindow.getInstance().appendLog("DEBUG: " + message + "\n");
        }
    }

    public static void debug(Throwable t) {
        if (DEBUG) {
            debug(getStackTraceString(t));
        }
    }

    public static void debugWithThread(String message) {
        if (DEBUG) {
            debug(Thread.currentThread().getName() + ": " + message);
        }
    }

    public static void debugWithThread(Throwable t) {
        if (DEBUG) {
            debugWithThread(getStackTraceString(t));
        }
    }

    public static void info(String message) {
        System.out.println(message);
        LogWindow.getInstance().appendLog(message + "\n");
    }

    public static void info(String format, Object... arg) {
        System.out.printf(format, arg);
    }

    public static void warning(String message) {
        System.out.println(message);
        LogWindow.getInstance().appendLog(message + "\n");
    }

    public static void error(Throwable t) {
        System.out.println(getStackTraceString(t));
        LogWindow.getInstance().appendLog(getStackTraceString(t) + "\n");
    }

    private static String getStackTraceString(Throwable t) {
        if (t == null) {
            return "null";
        }

        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        t.printStackTrace(pw);
        return sw.toString();
    }

}
