package dareka.processor.impl;

import java.io.IOException;
import java.io.OutputStream;
import java.util.concurrent.FutureTask;

import dareka.common.Logger;
import dareka.processor.HttpResponseHeader;
import dareka.processor.TransferListener;

public class NicoCachingListener implements TransferListener {
    private Cache cache;
    private FutureTask<String> retrieveTitleTask;
    private OutputStream out;
    private boolean errorOccured;

    public NicoCachingListener(Cache cache, FutureTask<String> getTitlteTask)
            throws IOException {
        this.cache = cache;
        this.retrieveTitleTask = getTitlteTask;
        out = cache.getTmpOutputStream();
        errorOccured = false;
    }

    public void onTransferring(byte[] buf, int length) {
        if (errorOccured) {
            return;
        }

        try {
            out.write(buf, 0, length);
        } catch (IOException e) {
            Logger.warning(cache.getCacheFileName() + ": " + e.toString());
            errorOccured = true;
        }
    }

    public void onTransferEnd(boolean completed) {

        try {
            out.close();
        } catch (IOException e) {
            Logger.debugWithThread(e);
            errorOccured = true;
        }

        try {
            if (errorOccured || completed == false) {
                cache.deleteTmp();
                Logger.debugWithThread(cache.getCacheFileName() + " deleted");
                
                retrieveTitleTask.cancel(true);
            } else {
                String title;
                
                try {
                    if (retrieveTitleTask != null && (title = retrieveTitleTask.get()) != null) {
                        cache.setDescribe(title);
                    }
                } catch (Exception e) {
                    Logger.warning("title retrieving failed: " + e.toString());
                }

                cache.store();
                Logger.info("cache completed: " + cache.getCacheFileName());
            }
        } catch (IOException e) {
            Logger.debugWithThread(e);
            Logger.warning(e.toString());
        }

    }

    public void onResonseHeader(HttpResponseHeader responseHeader) {
        if (responseHeader.getContentLength() <= 0) {
            errorOccured = true;
        }
    }
}