package dareka.processor.impl;

import java.io.IOException;
import java.util.concurrent.Executor;
import java.util.concurrent.FutureTask;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dareka.common.Config;
import dareka.common.Logger;
import dareka.processor.HttpRequestHeader;
import dareka.processor.Processor;
import dareka.processor.Resource;

public class NicoCachingProcessor implements Processor {
    private static final String[] SUPPORTED_METHODS = new String[] { "GET" };
    /**
     * SMILEVIDEO̓URL̐K\
     */
    // http://s-clb5.smilevideo.jp/smile?v=102982.92382
    private static final Pattern SM_FLV_PATTERN = Pattern
            .compile("^http://[^/]+(?:smilevideo|nicovideo)\\.jp/smile\\?v=([^.]+)\\.\\d+(low)?$");

    private Executor executor;
    private Config config;

    public NicoCachingProcessor(Executor executor, Config config) {
        this.executor = executor;
        this.config = config;
    }

    public String[] getSupportedMethods() {
        return SUPPORTED_METHODS;
    }

    public Pattern getSupportedURLAsPattern() {
        return SM_FLV_PATTERN;
    }

    public String getSupportedURLAsString() {
        return null;
    }

    public Resource onRequest(HttpRequestHeader requestHeader)
            throws IOException {
        Matcher m = SM_FLV_PATTERN.matcher(requestHeader.getURI());
        if (!m.find()) {
            // it must not happen...
            return Resource.get(Resource.Type.URL, requestHeader.getURI());
        }

        String smId = "sm" + m.group(1);
        String suffix = m.group(2);

        Cache cache = new Cache(smId);
        if (!cache.exists() && suffix != null) {
            cache = new Cache(smId + suffix);
        }

        if (cache.exists()) {
            Logger.info("using cache: " + cache.getCacheFileName());
            return Resource.get(Resource.Type.URL, cache.getURLString());
        }

        FutureTask<String> retrieveTitlteTask = null;
        if (config.isTitle()) {
            retrieveTitlteTask = new FutureTask<String>(
                    new NicoCachingTitleRetriever(smId, config));
            executor.execute(retrieveTitlteTask);
        }

        Logger.info("no cache found: " + cache.getCacheFileName());

        Resource r = Resource.get(Resource.Type.URL, requestHeader.getURI());
        r.addTransferListener(new NicoCachingListener(cache, retrieveTitlteTask));

        return r;
    }

}
