package dareka.processor.impl;

import java.io.ByteArrayOutputStream;
import java.util.concurrent.Callable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dareka.common.Config;
import dareka.common.Logger;
import dareka.processor.URLResource;

public class NicoCachingTitleRetriever implements Callable<String> {
    /**
     * ^Cg̐K\
     */
    private static final Pattern HTML_TITLE_PATTERN = Pattern
            .compile("<title>(?:jRjR\\([^-]]+[-]])?\\s*(.*?)</title>");

    private String smId;
    private Config config;

    public NicoCachingTitleRetriever(String smId, Config config) {
        this.smId = smId;
        this.config = config;
    }

    public String call() throws Exception {
        Logger.debugWithThread("title retrieving start");

        String url = "http://www.nicovideo.jp/thumb/" + smId;
        URLResource r = new URLResource(url);

        ByteArrayOutputStream bout = new ByteArrayOutputStream();
        r.transferTo(null, bout, null, config);

        String responseString = new String(bout.toByteArray(), "UTF-8");

        String title = getTitleFromResponse(responseString);
        Logger.debugWithThread("title retrieving end (" + title + ")");

        return title;
    }

    /**
     * HTML^Cg擾B
     * 
     * @param html
     * @return ^CgBȂꍇnullB
     */
    String getTitleFromResponse(String html) {
        Matcher mTitle = HTML_TITLE_PATTERN.matcher(html);
        if (mTitle.find()) {
            return mTitle.group(1);
        } else {
            return null;
        }
    }
}
